@@ -1,6 +1,6 @@
 /* gdkwindow-quartz.c
  *
- * Copyright (C) 2005 Imendio AB
+ * Copyright (C) 2005-2006 Imendio AB
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,8 +21,6 @@
 #ifndef __GDK_PRIVATE_QUARTZ_H__
 #define __GDK_PRIVATE_QUARTZ_H__
 
-#include <config.h>
-
 #define GDK_QUARTZ_ALLOC_POOL NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init]
 #define GDK_QUARTZ_RELEASE_POOL [pool release]
 
@@ -34,6 +32,8 @@
 
 #include "gdkinternals.h"
 
+#include <config.h>
+
 #define GDK_TYPE_GC_QUARTZ              (_gdk_gc_quartz_get_type ())
 #define GDK_GC_QUARTZ(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), GDK_TYPE_GC_QUARTZ, GdkGCQuartz))
 #define GDK_GC_QUARTZ_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), GDK_TYPE_GC_QUARTZ, GdkGCQuartzClass))
@@ -98,10 +98,10 @@ void _gdk_events_init           (void);
 void _gdk_visual_init           (void);
 void _gdk_input_init            (void);
 
-void _gdk_quartz_set_context_fill_color_from_pixel (CGContextRef context, GdkColormap *colormap, guint32 pixel);
-void _gdk_quartz_set_context_stroke_color_from_pixel (CGContextRef context, GdkColormap *colormap, guint32 pixel);
+void gdk_quartz_set_context_fill_color_from_pixel (CGContextRef context, GdkColormap *colormap, guint32 pixel);
+void gdk_quartz_set_context_stroke_color_from_pixel (CGContextRef context, GdkColormap *colormap, guint32 pixel);
 
-void _gdk_quartz_update_context_from_gc (CGContextRef context, GdkGC *gc);
+void gdk_quartz_update_context_from_gc (CGContextRef context, GdkGC *gc);
 
 gint        _gdk_quartz_get_inverted_screen_y      (gint y);
 
@@ -117,20 +117,22 @@ GdkWindow *_gdk_quartz_get_mouse_window     (void);
 void       _gdk_quartz_update_mouse_window  (GdkWindow *window);
 void       _gdk_quartz_update_cursor        (GdkWindow *window);
 
-GdkImage *_gdk_quartz_copy_to_image (GdkDrawable *drawable,
-				     GdkImage    *image,
-				     gint         src_x,
-				     gint         src_y,
-				     gint         dest_x,
-				     gint         dest_y,
-				     gint         width,
-				     gint         height);
+GdkImage  *_gdk_quartz_copy_to_image (GdkDrawable *drawable,
+				      GdkImage    *image,
+				      gint         src_x,
+				      gint         src_y,
+				      gint         dest_x,
+				      gint         dest_y,
+				      gint         width,
+				      gint         height);
 
 void _gdk_quartz_send_map_events (GdkWindow *window);
 
 GdkEventType _gdk_quartz_key_event_type  (NSEvent   *event);
 gboolean     _gdk_quartz_key_is_modifier (guint      keycode);
 
+GdkEventMask _gdk_quartz_get_current_event_mask (void);
+
 extern GdkWindow *_gdk_quartz_keyboard_grab_window;
 extern GdkWindow *_gdk_quartz_pointer_grab_window;
 
