@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-file-style: "gnu"; tab-width: 8 -*- */
 /* gtkpathbar.c
  * Copyright (C) 2004  Red Hat, Inc.,  Jonathan Blandford <jrb@gnome.org>
  *
@@ -75,7 +76,7 @@ struct _ButtonData
  */
 #define BUTTON_IS_FAKE_ROOT(button) ((button)->type == HOME_BUTTON)
 
-G_DEFINE_TYPE (GtkPathBar, gtk_path_bar, GTK_TYPE_CONTAINER);
+G_DEFINE_TYPE (GtkPathBar, gtk_path_bar, GTK_TYPE_CONTAINER)
 
 static void gtk_path_bar_finalize                 (GObject          *object);
 static void gtk_path_bar_dispose                  (GObject          *object);
@@ -116,6 +117,16 @@ static void gtk_path_bar_update_button_appearance (GtkPathBar       *path_bar,
 						   ButtonData       *button_data,
 						   gboolean          current_dir);
 
+static void
+on_slider_unmap (GtkWidget  *widget,
+		 GtkPathBar *path_bar)
+{
+  if (path_bar->timer &&
+      ((widget == path_bar->up_slider_button && path_bar->scrolling_up) ||
+       (widget == path_bar->down_slider_button && path_bar->scrolling_down)))
+    gtk_path_bar_stop_scrolling (path_bar);
+}
+
 static GtkWidget *
 get_slider_button (GtkPathBar  *path_bar,
 		   GtkArrowType arrow_type)
@@ -130,6 +141,9 @@ get_slider_button (GtkPathBar  *path_bar,
   gtk_container_add (GTK_CONTAINER (path_bar), button);
   gtk_widget_show_all (button);
 
+  g_signal_connect (G_OBJECT (button), "unmap",
+		    G_CALLBACK (on_slider_unmap), path_bar);
+
   gtk_widget_pop_composite_child ();
 
   return button;
@@ -250,6 +264,7 @@ remove_settings_signal (GtkPathBar *path_bar,
 static void
 gtk_path_bar_dispose (GObject *object)
 {
+  GList *list;
   GtkPathBar *path_bar = GTK_PATH_BAR (object);
 
   remove_settings_signal (path_bar, gtk_widget_get_screen (GTK_WIDGET (object)));
@@ -1010,6 +1025,12 @@ set_button_image_get_info_cb (GtkFileSystemHandle *handle,
 
   data->button_data->handle = NULL;
 
+  if (!data->button_data->button)
+    {
+      g_free (data->button_data);
+      goto out;
+    }
+
   if (cancelled || error)
     goto out;
 
@@ -1124,12 +1145,20 @@ set_button_image (GtkPathBar *path_bar,
 static void
 button_data_free (ButtonData *button_data)
 {
+  if (button_data->path)
+    gtk_file_path_free (button_data->path);
+  button_data->path = NULL;
+
+  if (button_data->dir_name)
+    g_free (button_data->dir_name);
+  button_data->dir_name = NULL;
+
+  button_data->button = NULL;
+
   if (button_data->handle)
     gtk_file_system_cancel_operation (button_data->handle);
-
-  gtk_file_path_free (button_data->path);
-  g_free (button_data->dir_name);
-  g_free (button_data);
+  else
+    g_free (button_data);
 }
 
 static const char *
@@ -1322,7 +1351,7 @@ make_directory_button (GtkPathBar  *path_bar,
 		       targets,
 		       G_N_ELEMENTS (targets),
 		       GDK_ACTION_COPY);
-  g_signal_connect (button_data->button, "drag-data-get",
+  g_signal_connect (button_data->button, "drag_data_get",
 		    G_CALLBACK (button_drag_data_get_cb), button_data);
 
   return button_data;
@@ -1643,3 +1672,6 @@ _gtk_path_bar_down (GtkPathBar *path_bar)
 	}
     }
 }
+
+#define __GTK_PATH_BAR_C__
+#include "gtkaliasdef.c"
