@@ -77,6 +77,7 @@
 #define GTK_TEXT_USE_INTERNAL_UNSUPPORTED_API
 #include <config.h>
 #include "gtktextdisplay.h"
+#include "gtkintl.h"
 #include "gtkalias.h"
 /* DO NOT go putting private headers in here. This file should only
  * use the semi-public headers, as with gtktextview.c.
@@ -122,7 +123,7 @@ text_renderer_get_error_color (GtkTextRenderer *text_renderer)
 
   if (!text_renderer->error_color)
     gtk_widget_style_get (text_renderer->widget,
-			  "error-underline_color", &text_renderer->error_color,
+			  "error-underline-color", &text_renderer->error_color,
 			  NULL);
   
   if (!text_renderer->error_color)
@@ -646,12 +647,13 @@ render_para (GtkTextRenderer    *text_renderer,
 
 static void
 on_renderer_display_closed (GdkDisplay       *display,
+                            gboolean          is_error,
 			    GtkTextRenderer  *text_renderer)
 {
   g_signal_handlers_disconnect_by_func (text_renderer->screen,
 					(gpointer)on_renderer_display_closed,
 					text_renderer);
-  g_object_set_data (G_OBJECT (text_renderer->screen), "gtk-text-renderer", NULL);
+  g_object_set_data (G_OBJECT (text_renderer->screen), I_("gtk-text-renderer"), NULL);
 }
 
 static GtkTextRenderer *
@@ -667,11 +669,12 @@ get_text_renderer (GdkScreen *screen)
       text_renderer = g_object_new (GTK_TYPE_TEXT_RENDERER, "screen", screen, NULL);
       text_renderer->screen = screen;
       
-      g_object_set_data_full (G_OBJECT (screen), "gtk-text-renderer", text_renderer,
+      g_object_set_data_full (G_OBJECT (screen), I_("gtk-text-renderer"), text_renderer,
 			      (GDestroyNotify)g_object_unref);
 
-      g_signal_connect (gdk_screen_get_display (screen), "closed",
-			G_CALLBACK (on_renderer_display_closed), text_renderer);
+      g_signal_connect_object (gdk_screen_get_display (screen), "closed",
+                               G_CALLBACK (on_renderer_display_closed),
+                               text_renderer, 0);
     }
 
   return text_renderer;
