@@ -565,6 +565,30 @@ gtk_im_context_xim_finalize (GObject *obj)
 
   context_xim->finalizing = TRUE;
 
+  if (context_xim->im_info && !context_xim->im_info->ics->next) 
+    {
+      if (context_xim->im_info->reconnecting)
+	{
+	  GdkDisplay *display;
+
+	  display = gdk_screen_get_display (context_xim->im_info->screen);
+	  XUnregisterIMInstantiateCallback (GDK_DISPLAY_XDISPLAY (display),
+					    NULL, NULL, NULL,
+					    xim_instantiate_callback,
+					    (XPointer)context_xim->im_info);
+	}
+      else if (context_xim->im_info->im)
+	{
+	  XIMCallback im_destroy_callback;
+
+	  im_destroy_callback.client_data = NULL;
+	  im_destroy_callback.callback = NULL;
+	  XSetIMValues (context_xim->im_info->im,
+			XNDestroyCallback, &im_destroy_callback,
+			NULL);
+	}
+    }
+
   set_ic_client_window (context_xim, NULL);
 
   g_free (context_xim->locale);
@@ -704,6 +728,10 @@ gtk_im_context_xim_filter_keypress (GtkIMContext *context,
   if (XFilterEvent ((XEvent *)&xevent, GDK_DRAWABLE_XID (context_xim->client_window)))
     return TRUE;
   
+  if (event->state &
+      (gtk_accelerator_get_default_mod_mask () & ~(GDK_SHIFT_MASK | GDK_CONTROL_MASK))) 
+    return FALSE;
+
  again:
   if (ic)
     num_bytes = XmbLookupString (ic, &xevent, buffer, buffer_size, &keysym, &status);
