@@ -20,6 +20,8 @@
  */
 
 #include "gtkcupsutils.h"
+#include "config.h"
+#include "gtkdebug.h"
 
 #include <errno.h>
 #include <unistd.h>
@@ -28,6 +30,10 @@
 #include <stdlib.h>
 #include <time.h>
 
+#if CUPS_VERSION_MAJOR > 1 || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR > 1) || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR == 1 && CUPS_VERSION_PATCH >= 20)
+#define HAVE_HTTP_AUTHSTRING 1
+#endif
+
 typedef void (*GtkCupsRequestStateFunc) (GtkCupsRequest *request);
 
 static void _connect            (GtkCupsRequest *request);
@@ -86,7 +92,7 @@ GtkCupsRequest *
 gtk_cups_request_new (http_t *connection,
                       GtkCupsRequestType req_type, 
                       gint operation_id,
-                      gint data_fd,
+                      GIOChannel *data_io,
                       const char *server,
                       const char *resource)
 {
@@ -135,7 +141,7 @@ gtk_cups_request_new (http_t *connection,
   request->last_status = HTTP_CONTINUE;
 
   request->attempts = 0;
-  request->data_fd = data_fd;
+  request->data_io = data_io;
 
   request->ipp_request = ippNew();
   request->ipp_request->request.op.operation_id = operation_id;
@@ -151,6 +157,10 @@ gtk_cups_request_new (http_t *connection,
                                    "attributes-natural-language", 
                                    NULL, language->language);
 
+  gtk_cups_request_ipp_add_string (request, IPP_TAG_OPERATION, IPP_TAG_NAME,
+                                   "requesting-user-name",
+                                   NULL, cupsUser());
+  
   cupsLangFree (language);
 
   return request;
@@ -351,10 +361,15 @@ _find_option_tag (const gchar *option)
     }
 }
 
+/*
+ * Note that this function uses IPP_TAG_JOB, so it is
+ * only suitable for IPP Group 2 attributes.
+ * See RFC 2911.
+ */
 void
 gtk_cups_request_encode_option (GtkCupsRequest *request,
-                                const gchar *option,
-				const gchar *value)
+                                const gchar    *option,
+			        const gchar    *value)
 {
   ipp_tag_t option_tag;
 
@@ -378,7 +393,7 @@ gtk_cups_request_encode_option (GtkCupsRequest *request,
       case IPP_TAG_INTEGER:
       case IPP_TAG_ENUM:
         ippAddInteger (request->ipp_request,
-                       IPP_TAG_OPERATION,
+                       IPP_TAG_JOB,
                        option_tag,
                        option,
                        strtol (value, NULL, 0));
@@ -394,7 +409,7 @@ gtk_cups_request_encode_option (GtkCupsRequest *request,
 	    b = 1;
 	  
           ippAddBoolean(request->ipp_request,
-                        IPP_TAG_OPERATION,
+                        IPP_TAG_JOB,
                         option,
                         b);
         
@@ -426,7 +441,7 @@ gtk_cups_request_encode_option (GtkCupsRequest *request,
 	    upper = lower;
          
           ippAddRange (request->ipp_request,
-                       IPP_TAG_OPERATION,
+                       IPP_TAG_JOB,
                        option,
                        lower,
                        upper);
@@ -454,7 +469,7 @@ gtk_cups_request_encode_option (GtkCupsRequest *request,
             units = IPP_RES_PER_INCH;
           
           ippAddResolution (request->ipp_request,
-                            IPP_TAG_OPERATION,
+                            IPP_TAG_JOB,
                             option,
                             units,
                             xres,
@@ -464,12 +479,80 @@ gtk_cups_request_encode_option (GtkCupsRequest *request,
         }
 
       default:
-        ippAddString (request->ipp_request,
-                      IPP_TAG_OPERATION,
-                      option_tag,
-                      option,
-                      NULL,
-                      value);
+        {
+          char *values;
+          char *s;
+          int in_quotes;
+          char *next;
+          GPtrArray *strings;
+          
+          values = g_strdup (value);
+          strings = NULL;
+          in_quotes = 0;
+          
+          for (s = values, next = s; *s != '\0'; s++)
+            {
+              if (in_quotes != 2 && *s == '\'')
+                {
+                  /* skip quoted value */
+                  if (in_quotes == 0)
+                    in_quotes = 1;
+                  else
+                    in_quotes = 0;
+                }
+              else if (in_quotes != 1 && *s == '\"')
+                {
+                  /* skip quoted value */
+                  if (in_quotes == 0)
+                    in_quotes = 2;
+                  else
+                    in_quotes = 0;
+                }
+              else if (in_quotes == 0 && *s == ',')
+                {
+                  /* found delimiter, add to value array */
+                  *s = '\0';
+                  if (strings == NULL)
+                    strings = g_ptr_array_new ();
+                  g_ptr_array_add (strings, next);
+                  next = s + 1;
+                }
+              else if (in_quotes == 0 && *s == '\\' && s[1] != '\0')
+                {
+                  /* skip escaped character */
+                  s++;
+                }
+            }
+          
+          if (strings == NULL)
+            {
+              /* single value */
+              ippAddString (request->ipp_request,
+                            IPP_TAG_JOB,
+                            option_tag,
+                            option,
+                            NULL,
+                            value);
+            }
+          else
+            {
+              /* multiple values */
+              
+              /* add last value */
+              g_ptr_array_add (strings, next);
+              
+              ippAddStrings (request->ipp_request,
+                             IPP_TAG_JOB,
+                             option_tag,
+                             option,
+                             strings->len,
+                             NULL,
+                             (const char **) strings->pdata);
+              g_ptr_array_free (strings, TRUE);
+            }
+
+          g_free (values);
+        }
 
         break;
     }
@@ -510,11 +593,14 @@ _post_send (GtkCupsRequest *request)
   gchar length[255];
   struct stat data_info;
 
+  GTK_NOTE (PRINTING,
+            g_print ("CUPS Backend: %s\n", G_STRFUNC));
+
   request->poll_state = GTK_CUPS_HTTP_WRITE;
 
-  if (request->data_fd != 0)
+  if (request->data_io != NULL)
     {
-      fstat (request->data_fd, &data_info);
+      fstat (g_io_channel_unix_get_fd (request->data_io), &data_info);
       sprintf (length, "%lu", (unsigned long)ippLength(request->ipp_request) + data_info.st_size);
     }
   else
@@ -525,7 +611,9 @@ _post_send (GtkCupsRequest *request)
   httpClearFields(request->http);
   httpSetField(request->http, HTTP_FIELD_CONTENT_LENGTH, length);
   httpSetField(request->http, HTTP_FIELD_CONTENT_TYPE, "application/ipp");
+#ifdef HAVE_HTTP_AUTHSTRING
   httpSetField(request->http, HTTP_FIELD_AUTHORIZATION, request->http->authstring);
+#endif
 
   if (httpPost(request->http, request->resource))
     {
@@ -552,6 +640,9 @@ _post_write_request (GtkCupsRequest *request)
 {
   ipp_state_t ipp_status;
 
+  GTK_NOTE (PRINTING,
+            g_print ("CUPS Backend: %s\n", G_STRFUNC));
+
   request->poll_state = GTK_CUPS_HTTP_WRITE;
   
   ipp_status = ippWrite(request->http, request->ipp_request);
@@ -567,7 +658,7 @@ _post_write_request (GtkCupsRequest *request)
 
   if (ipp_status == IPP_DATA)
     {
-      if (request->data_fd != 0)
+      if (request->data_io != NULL)
         request->state = GTK_CUPS_POST_WRITE_DATA;
       else
         {
@@ -580,10 +671,13 @@ _post_write_request (GtkCupsRequest *request)
 static void 
 _post_write_data (GtkCupsRequest *request)
 {
-  ssize_t bytes;
+  gsize bytes;
   char buffer[_GTK_CUPS_MAX_CHUNK_SIZE];
   http_status_t http_status;
 
+  GTK_NOTE (PRINTING,
+            g_print ("CUPS Backend: %s\n", G_STRFUNC));
+
   request->poll_state = GTK_CUPS_HTTP_WRITE;
   
   if (httpCheck (request->http))
@@ -596,10 +690,30 @@ _post_write_data (GtkCupsRequest *request)
 
   if (http_status == HTTP_CONTINUE || http_status == HTTP_OK)
     {
-      /* send data */
-      bytes = read(request->data_fd, buffer, _GTK_CUPS_MAX_CHUNK_SIZE);
+      GIOStatus io_status;
+      GError *error;
 
-      if (bytes == 0)
+      error = NULL;
+
+      /* send data */
+      io_status =
+        g_io_channel_read_chars (request->data_io, 
+	                         buffer, 
+				 _GTK_CUPS_MAX_CHUNK_SIZE,
+				 &bytes,
+				 &error);
+
+      if (io_status == G_IO_STATUS_ERROR)
+        {
+          request->state = GTK_CUPS_POST_DONE;
+	  request->poll_state = GTK_CUPS_HTTP_IDLE;
+     
+          gtk_cups_result_set_error (request->result, "Error reading from cache file: %s", error->message);
+
+	  g_error_free (error);
+          return;
+	}
+      else if (bytes == 0 && io_status == G_IO_STATUS_EOF)
         {
           request->state = GTK_CUPS_POST_CHECK;
 	  request->poll_state = GTK_CUPS_HTTP_READ;
@@ -607,16 +721,13 @@ _post_write_data (GtkCupsRequest *request)
           request->attempts = 0;
           return;
         }
-      else if (bytes == -1)
-        {
-          request->state = GTK_CUPS_POST_DONE;
-	  request->poll_state = GTK_CUPS_HTTP_IDLE;
-     
-          gtk_cups_result_set_error (request->result, "Error reading from cache file: %s", strerror (errno));
-          return;
-	}
-	
-      if (httpWrite(request->http, buffer, (int)bytes) < bytes)
+
+
+#if HAVE_CUPS_API_1_2
+      if (httpWrite2(request->http, buffer, bytes) < bytes)
+#else
+      if (httpWrite(request->http, buffer, (int) bytes) < bytes)
+#endif /* HAVE_CUPS_API_1_2 */
         {
           request->state = GTK_CUPS_POST_DONE;
 	  request->poll_state = GTK_CUPS_HTTP_IDLE;
@@ -638,6 +749,9 @@ _post_check (GtkCupsRequest *request)
 
   http_status = request->last_status;
 
+  GTK_NOTE (PRINTING,
+            g_print ("CUPS Backend: %s - status %i\n", G_STRFUNC, http_status));
+
   request->poll_state = GTK_CUPS_HTTP_READ;
 
   if (http_status == HTTP_CONTINUE)
@@ -656,12 +770,11 @@ _post_check (GtkCupsRequest *request)
     }
   else if (http_status == HTTP_ERROR)
     {
+      int error = httpError (request->http);
 #ifdef G_OS_WIN32
-      if (request->http->error != WSAENETDOWN && 
-          request->http->error != WSAENETUNREACH)
+      if (error != WSAENETDOWN && error != WSAENETUNREACH)
 #else
-      if (request->http->error != ENETDOWN && 
-          request->http->error != ENETUNREACH)
+      if (error != ENETDOWN && error != ENETUNREACH)	  
 #endif /* G_OS_WIN32 */
         {
           request->attempts++;
@@ -740,6 +853,9 @@ _post_read_response (GtkCupsRequest *request)
 {
   ipp_state_t ipp_status;
 
+  GTK_NOTE (PRINTING,
+            g_print ("CUPS Backend: %s\n", G_STRFUNC));
+
   request->poll_state = GTK_CUPS_HTTP_READ;
 
   if (request->result->ipp_response == NULL)
@@ -768,11 +884,14 @@ _post_read_response (GtkCupsRequest *request)
 static void 
 _get_send (GtkCupsRequest *request)
 {
+  GTK_NOTE (PRINTING,
+            g_print ("CUPS Backend: %s\n", G_STRFUNC));
+
   request->poll_state = GTK_CUPS_HTTP_WRITE;
 
-  if (request->data_fd == 0)
+  if (request->data_io == NULL)
     {
-      gtk_cups_result_set_error (request->result, "Get requires an open file descriptor");
+      gtk_cups_result_set_error (request->result, "Get requires an open io channel");
       request->state = GTK_CUPS_GET_DONE;
       request->poll_state = GTK_CUPS_HTTP_IDLE;
 
@@ -780,7 +899,9 @@ _get_send (GtkCupsRequest *request)
     }
 
   httpClearFields(request->http);
+#ifdef HAVE_HTTP_AUTHSTRING
   httpSetField(request->http, HTTP_FIELD_AUTHORIZATION, request->http->authstring);
+#endif
 
   if (httpGet(request->http, request->resource))
     {
@@ -809,6 +930,9 @@ _get_check (GtkCupsRequest *request)
 {
   http_status_t http_status;
 
+  GTK_NOTE (PRINTING,
+            g_print ("CUPS Backend: %s\n", G_STRFUNC));
+
   http_status = request->last_status;
 
   request->poll_state = GTK_CUPS_HTTP_READ;
@@ -890,12 +1014,27 @@ static void
 _get_read_data (GtkCupsRequest *request)
 {
   char buffer[_GTK_CUPS_MAX_CHUNK_SIZE];
-  int bytes;
- 
-  request->poll_state = GTK_CUPS_HTTP_READ;
- 
-  bytes = httpRead(request->http, buffer, sizeof(buffer));
+  gsize bytes;
+  gsize bytes_written;
+  GIOStatus io_status;
+  GError *error;
 
+  GTK_NOTE (PRINTING,
+            g_print ("CUPS Backend: %s\n", G_STRFUNC));
+
+  error = NULL;
+
+  request->poll_state = GTK_CUPS_HTTP_READ;
+
+#if HAVE_CUPS_API_1_2
+  bytes = httpRead2(request->http, buffer, sizeof(buffer));
+#else
+  bytes = httpRead(request->http, buffer, sizeof(buffer));
+#endif /* HAVE_CUPS_API_1_2 */
+
+  GTK_NOTE (PRINTING,
+            g_print ("CUPS Backend: %i bytes read\n", bytes));
+  
   if (bytes == 0)
     {
       request->state = GTK_CUPS_GET_DONE;
@@ -903,16 +1042,21 @@ _get_read_data (GtkCupsRequest *request)
 
       return;
     }
-    
-  if (write (request->data_fd, buffer, bytes) == -1)
-    {
-      char *error_msg;
+  
+  io_status =
+    g_io_channel_write_chars (request->data_io, 
+                              buffer, 
+			      bytes, 
+			      &bytes_written,
+			      &error);
 
+  if (io_status == G_IO_STATUS_ERROR)
+    {
       request->state = GTK_CUPS_POST_DONE;
       request->poll_state = GTK_CUPS_HTTP_IDLE;
     
-      error_msg = strerror (errno);
-      gtk_cups_result_set_error (request->result, error_msg ? error_msg:""); 
+      gtk_cups_result_set_error (request->result, error->message);
+      g_error_free (error);
     }
 }
 
