@@ -49,6 +49,7 @@
 #include "gtkcupsutils.h"
 #include "gtkdebug.h"
 
+
 typedef struct _GtkPrintBackendCupsClass GtkPrintBackendCupsClass;
 
 #define GTK_PRINT_BACKEND_CUPS_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_PRINT_BACKEND_CUPS, GtkPrintBackendCupsClass))
@@ -269,7 +270,7 @@ _cairo_write_to_cups (void                *closure,
 
   while (length > 0) 
     {
-      g_io_channel_write_chars (io, data, length, &written, &error);
+      g_io_channel_write_chars (io, (gchar *)data, length, &written, &error);
 
       if (error != NULL)
 	{
@@ -421,9 +422,6 @@ gtk_print_backend_cups_print_stream (GtkPrintBackend         *print_backend,
   gtk_cups_request_ipp_add_string (request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri",
                                    NULL, cups_printer->printer_uri);
 
-  gtk_cups_request_ipp_add_string (request, IPP_TAG_OPERATION, IPP_TAG_NAME, "requesting-user-name",
-                                   NULL, cupsUser());
-
   title = gtk_print_job_get_title (job);
   if (title)
     gtk_cups_request_ipp_add_string (request, IPP_TAG_OPERATION, IPP_TAG_NAME, "job-name", NULL,
@@ -448,7 +446,8 @@ gtk_print_backend_cups_print_stream (GtkPrintBackend         *print_backend,
 static void
 gtk_print_backend_cups_init (GtkPrintBackendCups *backend_cups)
 {
-  backend_cups->list_printers_poll = 0;  
+  backend_cups->list_printers_poll = FALSE;  
+  backend_cups->got_default_printer = FALSE;  
   backend_cups->list_printers_pending = FALSE;
 
   cups_request_default_printer (backend_cups);
@@ -501,27 +500,29 @@ cups_dispatch_watch_check (GSource *source)
   dispatch = (GtkPrintCupsDispatchWatch *) source;
 
   poll_state = gtk_cups_request_get_poll_state (dispatch->request);
-  
-  if (dispatch->data_poll == NULL && 
-      dispatch->request->http != NULL)
-    {
-      dispatch->data_poll = g_new0 (GPollFD, 1);
-      dispatch->data_poll->fd = dispatch->request->http->fd;
 
-      g_source_add_poll (source, dispatch->data_poll);
-    }
-            
-  if (dispatch->data_poll != NULL && dispatch->request->http != NULL)
+  if (dispatch->request->http != NULL)
     {
-      if (dispatch->data_poll->fd != dispatch->request->http->fd)
-        dispatch->data_poll->fd = dispatch->request->http->fd;
-
-      if (poll_state == GTK_CUPS_HTTP_READ)
-        dispatch->data_poll->events = G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_PRI;
-      else if (poll_state == GTK_CUPS_HTTP_WRITE)
-        dispatch->data_poll->events = G_IO_OUT | G_IO_ERR;
+      if (dispatch->data_poll == NULL)
+	{
+	  dispatch->data_poll = g_new0 (GPollFD, 1);
+	  g_source_add_poll (source, dispatch->data_poll);
+	}
       else
-        dispatch->data_poll->events = 0;
+	{
+	  if (poll_state == GTK_CUPS_HTTP_READ)
+	    dispatch->data_poll->events = G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_PRI;
+	  else if (poll_state == GTK_CUPS_HTTP_WRITE)
+	    dispatch->data_poll->events = G_IO_OUT | G_IO_ERR;
+	  else
+	    dispatch->data_poll->events = 0;
+	}
+
+#ifdef HAVE_CUPS_API_1_2
+      dispatch->data_poll->fd = httpGetFd (dispatch->request->http);
+#else
+      dispatch->data_poll->fd = dispatch->request->http->fd;
+#endif
     }
     
   if (poll_state != GTK_CUPS_HTTP_IDLE)  
@@ -964,7 +965,8 @@ cups_request_printer_list_cb (GtkPrintBackendCups *cups_backend,
   if (gtk_cups_result_is_error (result))
     {
       g_warning ("Error getting printer list: %s", gtk_cups_result_get_error_string (result));
-      return;
+
+      goto done;
     }
   
   /* Gather the names of the printers in the current queue
@@ -1074,11 +1076,11 @@ cups_request_printer_list_cb (GtkPrintBackendCups *cups_backend,
 			resource);
 #endif
 
-          if (member_uris && !strncmp (resource, "/printers/", 10))
+          if (!strncmp (resource, "/printers/", 10))
 	    {
 	      cups_printer->ppd_name = g_strdup (resource + 10);
               GTK_NOTE (PRINTING,
-                        g_print ("CUPS Backend: Setting ppd name '%s' for printer class '%s'\n", cups_printer->ppd_name, printer_name));
+                        g_print ("CUPS Backend: Setting ppd name '%s' for printer/class '%s'\n", cups_printer->ppd_name, printer_name));
             }
 
 	  gethostname (uri, sizeof(uri));
@@ -1133,6 +1135,8 @@ cups_request_printer_list_cb (GtkPrintBackendCups *cups_backend,
       list_has_changed = TRUE;
     }
   
+ done:
+
   if (list_has_changed)
     g_signal_emit_by_name (backend, "printer-list-changed");
   
@@ -1185,7 +1189,7 @@ cups_get_printer_list (GtkPrintBackend *backend)
   if (cups_backend->list_printers_poll == 0)
     {
       cups_request_printer_list (cups_backend);
-      cups_backend->list_printers_poll = g_timeout_add (3000 * 100000,
+      cups_backend->list_printers_poll = g_timeout_add (3000,
                                                         (GSourceFunc) cups_request_printer_list,
                                                         backend);
     }
@@ -1194,6 +1198,7 @@ cups_get_printer_list (GtkPrintBackend *backend)
 typedef struct {
   GtkPrinterCups *printer;
   GIOChannel *ppd_io;
+  http_t *http;
 } GetPPDData;
 
 static void
@@ -1201,7 +1206,7 @@ get_ppd_data_free (GetPPDData *data)
 {
   GTK_NOTE (PRINTING,
             g_print ("CUPS Backend: %s\n", G_STRFUNC));
-
+  httpClose (data->http);
   g_io_channel_unref (data->ppd_io);
   g_object_unref (data->printer);
   g_free (data);
@@ -1257,11 +1262,10 @@ cups_request_ppd (GtkPrinter *printer)
   GTK_NOTE (PRINTING,
             g_print ("CUPS Backend: %s\n", G_STRFUNC));
 
-  /* FIXME this can return NULL! */
   http = httpConnectEncrypt (cups_printer->hostname, 
 			     cups_printer->port,
 			     cupsEncryption ());
-
+  
   data = g_new0 (GetPPDData, 1);
 
   fd = g_file_open_tmp ("gtkprint_ppd_XXXXXX", 
@@ -1288,6 +1292,7 @@ cups_request_ppd (GtkPrinter *printer)
       return;
     }
     
+  data->http = http;
   fchmod (fd, S_IRUSR | S_IWUSR);
   data->ppd_io = g_io_channel_unix_new (fd);
   g_io_channel_set_encoding (data->ppd_io, NULL, NULL);
@@ -1296,8 +1301,9 @@ cups_request_ppd (GtkPrinter *printer)
   data->printer = g_object_ref (printer);
 
   resource = g_strdup_printf ("/printers/%s.ppd", 
-                              gtk_printer_cups_get_ppd_name (GTK_PRINTER_CUPS(printer)));
-  request = gtk_cups_request_new (http,
+                              gtk_printer_cups_get_ppd_name (GTK_PRINTER_CUPS (printer)));
+
+  request = gtk_cups_request_new (data->http,
                                   GTK_CUPS_GET,
 				  0,
                                   data->ppd_io,
@@ -1661,6 +1667,16 @@ value_is_off (const char *value)
 	   strcasecmp (value, "False") == 0);
 }
 
+static char *
+ppd_group_name (ppd_group_t *group)
+{
+#if CUPS_VERSION_MAJOR > 1 || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR > 1) || (CUPS_VERSION_MAJOR == 1 && CUPS_VERSION_MINOR == 1 && CUPS_VERSION_PATCH >= 18) 
+  return group->name;
+#else
+  return group->text;
+#endif
+}
+
 static int
 available_choices (ppd_file_t     *ppd,
 		   ppd_option_t   *option,
@@ -1686,7 +1702,10 @@ available_choices (ppd_file_t     *ppd,
   installed_options = NULL;
   for (i = 0; i < ppd->num_groups; i++)
     {
-      if (strcmp (ppd->groups[i].name, "InstallableOptions") == 0)
+      char *name; 
+
+      name = ppd_group_name (&ppd->groups[i]);
+      if (strcmp (name, "InstallableOptions") == 0)
 	{
 	  installed_options = &ppd->groups[i];
 	  break;
@@ -1763,12 +1782,13 @@ available_choices (ppd_file_t     *ppd,
 	all_default = FALSE;
     }
 
-  if (all_default && !keep_if_only_one_option)
-    return 0;
-  
-  if (num_conflicts == option->num_choices)
-    return 0;
+  if ((all_default && !keep_if_only_one_option) ||
+      (num_conflicts == option->num_choices))
+    {
+      g_free (conflicts);
 
+      return 0;
+    }
 
   /* Some ppds don't have a "use printer default" option for
    * InputSlot. This means you always have to select a particular slot,
@@ -1807,7 +1827,6 @@ available_choices (ppd_file_t     *ppd,
   
   if (available)
     {
-      
       *available = g_new (ppd_choice_t *, option->num_choices - num_conflicts + add_auto);
 
       i = 0;
@@ -1820,22 +1839,25 @@ available_choices (ppd_file_t     *ppd,
       if (add_auto) 
 	(*available)[i++] = NULL;
     }
+
+  g_free (conflicts);
   
   return option->num_choices - num_conflicts + add_auto;
 }
 
-#if HAVE_CUPS_API_1_2
 static GtkPrinterOption *
-create_pickone_option_custom (ppd_file_t   *ppd_file,
-		              ppd_option_t *ppd_option,
-			      ppd_coption_t *coption,
-		              const gchar  *gtk_name)
+create_pickone_option (ppd_file_t   *ppd_file,
+		       ppd_option_t *ppd_option,
+		       const gchar  *gtk_name)
 {
   GtkPrinterOption *option;
   ppd_choice_t **available;
   char *label;
   int n_choices;
   int i;
+#ifdef HAVE_CUPS_API_1_2
+  ppd_coption_t *coption;
+#endif
 
   g_assert (ppd_option->ui == PPD_UI_PICKONE);
   
@@ -1844,15 +1866,20 @@ create_pickone_option_custom (ppd_file_t   *ppd_file,
   n_choices = available_choices (ppd_file, ppd_option, &available, g_str_has_prefix (gtk_name, "gtk-"));
   if (n_choices > 0)
     {
-      ppd_cparam_t *cparam;
       
-      label = get_option_text (ppd_file, ppd_option);
-
       /* right now only support one parameter per custom option 
        * if more than one print warning and only offer the default choices
        */
+
+      label = get_option_text (ppd_file, ppd_option);
+
+#ifdef HAVE_CUPS_API_1_2
+      coption = ppdFindCustomOption (ppd_file, ppd_option->keyword);
+
       if (coption)
         {
+	  ppd_cparam_t *cparam;
+
           cparam = ppdFirstCustomParam (coption);
 
           if (ppdNextCustomParam (coption) == NULL)
@@ -1893,6 +1920,7 @@ create_pickone_option_custom (ppd_file_t   *ppd_file,
 	  else
 	    g_warning ("Not Supported: PPD Custom Option has more than one parameter");
 	}
+#endif /* HAVE_CUPS_API_1_2 */
 
       if (!option)
         option = gtk_printer_option_new (gtk_name, label,
@@ -1924,19 +1952,6 @@ create_pickone_option_custom (ppd_file_t   *ppd_file,
 
   return option;
 }
-#endif /* HAVE_CUPS_API_1_2 */
-
-static GtkPrinterOption *
-create_pickone_option (ppd_file_t   *ppd_file,
-		       ppd_option_t *ppd_option,
-		       const gchar  *gtk_name)
-{
-  GtkPrinterOption *option;  
-
-  option = create_pickone_option_custom (ppd_file, ppd_option, NULL, gtk_name);
-
-  return option;
-}
 
 static GtkPrinterOption *
 create_boolean_option (ppd_file_t   *ppd_file,
@@ -2019,11 +2034,6 @@ handle_option (GtkPrinterOptionSet *set,
   GtkPrinterOption *option;
   char *name;
 
-#ifdef HAVE_CUPS_API_1_2
-  ppd_coption_t *coption = NULL;
-  coption = ppdFindCustomOption (ppd_file, ppd_option->keyword);
-#endif
-
   if (STRING_IN_TABLE (ppd_option->keyword, cups_option_blacklist))
     return;
 
@@ -2032,12 +2042,7 @@ handle_option (GtkPrinterOptionSet *set,
   option = NULL;
   if (ppd_option->ui == PPD_UI_PICKONE)
     {
-#ifdef HAVE_CUPS_API_1_2
-      if (coption)
-        option = create_pickone_option_custom (ppd_file, ppd_option, coption, name);
-      else
-#endif
-        option = create_pickone_option (ppd_file, ppd_option, name);
+      option = create_pickone_option (ppd_file, ppd_option, name);
     }
   else if (ppd_option->ui == PPD_UI_BOOLEAN)
     {
@@ -2049,21 +2054,24 @@ handle_option (GtkPrinterOptionSet *set,
   
   if (option)
     {
-      if (STRING_IN_TABLE (toplevel_group->name,
+      char *name;
+
+      name = ppd_group_name (toplevel_group);
+      if (STRING_IN_TABLE (name,
 			   color_group_whitelist) ||
 	  STRING_IN_TABLE (ppd_option->keyword,
 			   color_option_whitelist))
 	{
 	  option->group = g_strdup ("ColorPage");
 	}
-      else if (STRING_IN_TABLE (toplevel_group->name,
+      else if (STRING_IN_TABLE (name,
 				image_quality_group_whitelist) ||
 	       STRING_IN_TABLE (ppd_option->keyword,
 				image_quality_option_whitelist))
 	{
 	  option->group = g_strdup ("ImageQualityPage");
 	}
-      else if (STRING_IN_TABLE (toplevel_group->name,
+      else if (STRING_IN_TABLE (name,
 				finishing_group_whitelist) ||
 	       STRING_IN_TABLE (ppd_option->keyword,
 				finishing_option_whitelist))
@@ -2091,9 +2099,11 @@ handle_group (GtkPrinterOptionSet *set,
 	      GtkPrintSettings    *settings)
 {
   gint i;
-
+  gchar *name;
+  
   /* Ignore installable options */
-  if (strcmp (toplevel_group->name, "InstallableOptions") == 0)
+  name = ppd_group_name (toplevel_group);
+  if (strcmp (name, "InstallableOptions") == 0)
     return;
   
   for (i = 0; i < group->num_options; i++)
@@ -2190,14 +2200,27 @@ cups_printer_get_options (GtkPrinter           *printer,
     {
       GtkPaperSize *paper_size;
       ppd_option_t *option;
+      const gchar  *ppd_name;
 
       ppdMarkDefaults (ppd_file);
 
       paper_size = gtk_page_setup_get_paper_size (page_setup);
 
       option = ppdFindOption (ppd_file, "PageSize");
-      strncpy (option->defchoice, gtk_paper_size_get_ppd_name (paper_size),
-	       PPD_MAX_NAME);
+      ppd_name = gtk_paper_size_get_ppd_name (paper_size);
+      
+      if (ppd_name)
+	strncpy (option->defchoice, ppd_name, PPD_MAX_NAME);
+      else
+        {
+          gchar *custom_name;
+
+	  custom_name = g_strdup_printf ("Custom.%2fx%.2f",
+					 gtk_paper_size_get_width (paper_size, GTK_UNIT_POINTS),
+					 gtk_paper_size_get_height (paper_size, GTK_UNIT_POINTS));
+          strncpy (option->defchoice, custom_name, PPD_MAX_NAME);
+          g_free (custom_name);
+        }
 
       for (i = 0; i < ppd_file->num_groups; i++)
         handle_group (set, ppd_file, &ppd_file->groups[i], &ppd_file->groups[i], settings);
